#include "StdAfx.h"
#include "logger.h"


CLogger::CLogger()
	: m_hFile(INVALID_HANDLE_VALUE)
{
}


CLogger::~CLogger()
{
	Release();
}


BOOL CLogger::Initialize(LPCSTR pszDirPath, LPCSTR pszFileTitle)
{
	Release();

	if (!MakeDirectory(pszDirPath))
		return FALSE;

	SYSTEMTIME st;
	char szFileName[MAX_PATH];
	char szFullPath[MAX_PATH];

	GetLocalTime(&st);
	sprintf(szFileName, "%s_%04d%02d%02d.txt", 
		pszFileTitle, st.wYear, st.wMonth, st.wDay );
	PathCombineA(szFullPath, pszDirPath, szFileName);

	m_hFile = CreateFileA(szFullPath,
		GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ,
		NULL, OPEN_ALWAYS,
		0, NULL);
	if (m_hFile == INVALID_HANDLE_VALUE)
		return FALSE;

	SetFilePointer (m_hFile, 0, NULL, FILE_END);

	return TRUE;
}


void CLogger::Release()
{
	if (m_hFile == INVALID_HANDLE_VALUE)
		return;

	CloseHandle(m_hFile);
	m_hFile = INVALID_HANDLE_VALUE;
}


void CLogger::Write(LPCSTR pszFormat, ...)
{
	if (m_hFile == INVALID_HANDLE_VALUE)
		return;

	va_list ap;
	DWORD dwBytesToWrite;
	DWORD dwBytesWritten;

	va_start(ap, pszFormat);
	dwBytesToWrite = vsprintf(m_szBuff, pszFormat, ap);
	va_end(ap);

	WriteFile(m_hFile, m_szBuff, dwBytesToWrite, (LPDWORD)&dwBytesWritten, NULL);
}


//void CLogger::Write(LPCSTR pszCallName, DWORD dwError)
//{
//	if (m_hFile == INVALID_HANDLE_VALUE)
//		return;
//
//	PSTR pszMessage = NULL;
//	FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
//		NULL, dwError, 0, (PSTR)&pszMessage, 0, NULL);
//
//	if (pszMessage != NULL)
//	{
//		DWORD dwBytesToWrite;
//		DWORD dwBytesWritten;
//		dwBytesToWrite = sprintf(m_szBuff, 
//			"Error: Failed to call %s : %d, %s\r\n", pszCallName, dwError, pszMessage);
//		WriteFile(m_hFile, m_szBuff, dwBytesToWrite, (LPDWORD)&dwBytesWritten, NULL);
//		LocalFree(pszMessage);
//	}
//}


void CLogger::Write(LPCSTR pszTag, LPBYTE pbBuff, DWORD dwLength)
{
	if (m_hFile == INVALID_HANDLE_VALUE)
		return;

	PSTR pstr;
	DWORD dwBytesToWrite;
	DWORD dwBytesWritten;

	SYSTEMTIME st;
	GetLocalTime(&st);

	dwBytesToWrite = sprintf(m_szBuff, 
		"%04d-%02d-%02d %02d:%02d:%02d:%03d  %s  ", 
		st.wYear, st.wMonth, st.wDay, st.wHour, 
		st.wMinute, st.wSecond, st.wMilliseconds, pszTag);
	WriteFile(m_hFile, m_szBuff, dwBytesToWrite, (LPDWORD)&dwBytesWritten, NULL);

	dwBytesToWrite = BytesToHexString(pbBuff, dwLength, pstr);
	if (dwBytesToWrite > 0)
	{
		WriteFile(m_hFile, pstr, dwBytesToWrite, (LPDWORD)&dwBytesWritten, NULL);
		free(pstr);
	}

	pstr = "\r\n";
	WriteFile(m_hFile, pstr, 2, (LPDWORD)&dwBytesWritten, NULL);
}


BOOL CLogger::MakeDirectory(LPCSTR pszDirPath)
{
	if (pszDirPath == NULL)
		return FALSE;

	if (PathFileExistsA(pszDirPath))
		return TRUE;

	SECURITY_ATTRIBUTES attrib;
	attrib.bInheritHandle = FALSE;
	attrib.lpSecurityDescriptor = NULL;
	attrib.nLength = sizeof(SECURITY_ATTRIBUTES);

	char szPath[MAX_PATH];
	LPSTR p = szPath;
	LPCSTR q = pszDirPath;

	while (1)
	{
		*p = *q;
		if (*p == '\\' || *p == '\0')
		{
			*p = 0;
			if (!PathFileExistsA(szPath))
			{
				if (!CreateDirectoryA(szPath, &attrib))
					return FALSE;
			}
			*p = '\\';

			if (*p == 0)
				break;
		}
		p++;
		q++;
	}

	return TRUE;
}


DWORD CLogger::BytesToHexString(LPBYTE pbBuff, DWORD dwLength, PSTR &pstr)
{
	size_t size = dwLength * 3 + 1;
	pstr = (PSTR)malloc(size);
	if (pstr == NULL)
		return 0;

	for (DWORD i = 0; i < dwLength; i++)
	{
		pstr[i * 3] = HexChar(pbBuff[i] / 16);
		pstr[i * 3 + 1] = HexChar(pbBuff[i] % 16);
		pstr[i * 3 + 2] = ' ';
	}

	pstr[dwLength * 3] = 0;
	return size - 1;
}


char CLogger::HexChar(int digit)
{
	if (digit <= 9)
		return '0' + digit;
	else
		return 'A' + (digit - 10);
}