// CiccPanel.cpp : ʵļ
//

#include "stdafx.h"
#include "F3TEST.h"
#include "CiccWorkspace.h"

enum ACTIVATESEQ
{
	AS_TYPE_A_THEN_TYPE_B,
	AS_TYPE_B_THEN_TYPE_A,
	AS_TYPE_A_ONLY,
	AS_TYPE_B_ONLY
};

// CCiccWorkspace Ի

IMPLEMENT_DYNAMIC(CCiccWorkspace, CPanelBase)

CCiccWorkspace::CCiccWorkspace(CWnd* pParent /*=NULL*/)
	: CPanelBase(CCiccWorkspace::IDD, pParent)
{

}

CCiccWorkspace::~CCiccWorkspace()
{
}

void CCiccWorkspace::DoDataExchange(CDataExchange* pDX)
{
	CPanelBase::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CICC_TAB, m_tab);
	DDX_Control(pDX, IDC_RFC_ACTIVATION_SEQUENCES, m_ActivationSequences);
}


BEGIN_MESSAGE_MAP(CCiccWorkspace, CPanelBase)
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_BN_CLICKED(IDC_CICC_ACTIVATE, &CCiccWorkspace::OnBnClickedCiccActivate)
	ON_BN_CLICKED(IDC_CICC_DEACTIVATE, &CCiccWorkspace::OnBnClickedCiccDeactivate)
	ON_BN_CLICKED(IDC_CICC_STATUS, &CCiccWorkspace::OnBnClickedCiccStatus)
	ON_WM_ENABLE()
	ON_CBN_SELCHANGE(IDC_RFC_ACTIVATION_SEQUENCES, &CCiccWorkspace::OnCbnSelchangeRfcActivationSequences)
	ON_BN_CLICKED(IDC_ISO14443_3_ACT, &CCiccWorkspace::OnBnClickedIso144433Act)
	ON_BN_CLICKED(IDC_ISO14443_4_ACT, &CCiccWorkspace::OnBnClickedIso144434Act)
END_MESSAGE_MAP()


// CCiccWorkspace Ϣ


BOOL CCiccWorkspace::OnInitDialog()
{
	CPanelBase::OnInitDialog();
	
	m_tab.AddPage( _T("Mifare S50"), m_pS50Wnd);
	m_tab.AddPage( _T("Mifare S70"), m_pS70Wnd);
	m_tab.AddPage( _T("Mifare Ultralight"), m_pUlWnd);
	m_tab.AddPage( _T("Type A CPU"), m_pTypeACpuWnd);
	m_tab.AddPage( _T("Type B CPU"), m_pTypeBCpuWnd);
	m_tab.AddPage( _T("ISO15693 RFcard"), m_pIsoWnd);
	

	int idx;
	idx = m_ActivationSequences.AddString(_T("TypeATypeB"));
	m_ActivationSequences.SetItemData(idx, AS_TYPE_A_THEN_TYPE_B);
	idx = m_ActivationSequences.AddString(_T("TypeB, TypeA"));
	m_ActivationSequences.SetItemData(idx, AS_TYPE_B_THEN_TYPE_A);
	idx = m_ActivationSequences.AddString(_T("TypeA"));
	m_ActivationSequences.SetItemData(idx, AS_TYPE_A_ONLY);
	idx = m_ActivationSequences.AddString(_T("TypeB"));
	m_ActivationSequences.SetItemData(idx, AS_TYPE_B_ONLY);
	m_ActivationSequences.SetCurSel(0);


	return TRUE;  // return TRUE unless you set the focus to a control
	// 쳣: OCX ҳӦ FALSE
}


void CCiccWorkspace::OnSize(UINT nType, int cx, int cy)
{
	CPanelBase::OnSize(nType, cx, cy);

	if (m_tab)
	{
		RECT rect;
		m_tab.GetWindowRect(&rect);
		ScreenToClient(&rect);
		rect.right = rect.left + cx;
		rect.bottom = rect.top + cy;
		m_tab.MoveWindow(&rect);
	}
}

int CCiccWorkspace::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CPanelBase::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_pS50Wnd = new CSxxPanel(S50);
	if (! m_pS50Wnd->Create(this))
		return -1;

	m_pS70Wnd = new CSxxPanel(S70);
	if (! m_pS70Wnd->Create(this))
		return -1;

	m_pUlWnd = new CUlPanel();
	if (! m_pUlWnd->Create(this))
		return -1;

	m_pTypeACpuWnd = new CConCpuPanel(TYPE_A_CPU);
	if (! m_pTypeACpuWnd->Create(this))
		return -1;

	m_pTypeBCpuWnd = new CConCpuPanel(TYPE_B_CPU);
	if (! m_pTypeBCpuWnd->Create(this))
		return -1;

	m_pIsoWnd = new CIsoPanel();
	if (! m_pIsoWnd->Create(this))
		return -1;
	

	return 0;
}

void CCiccWorkspace::OnBnClickedCiccActivate()
{
	BYTE ATRBuff[32];
	DWORD ATRLength = sizeof(ATRBuff);

	ACTIVATESEQ as = (ACTIVATESEQ) m_ActivationSequences.GetItemData(m_ActivationSequences.GetCurSel());
	
	LONG lResult = 0;
	switch (as )
	{
	case AS_TYPE_A_THEN_TYPE_B:
		lResult = F3_RfcActivate(g_hReader, ATRBuff, &ATRLength, RFC_PROTOCOL_TYPE_A, RFC_PROTOCOL_TYPE_B);//0x41,0x42
		break;
	case AS_TYPE_B_THEN_TYPE_A:
		lResult = F3_RfcActivate(g_hReader, ATRBuff, &ATRLength, RFC_PROTOCOL_TYPE_B, RFC_PROTOCOL_TYPE_A);//0x42,0x41
		break;
	case AS_TYPE_A_ONLY:
		lResult = F3_RfcActivate(g_hReader, ATRBuff, &ATRLength, RFC_PROTOCOL_TYPE_A);//0x41
		break;
	case AS_TYPE_B_ONLY:
		lResult = F3_RfcActivate(g_hReader, ATRBuff, &ATRLength, RFC_PROTOCOL_TYPE_B); //0x42
		break;
	}
	CString strMsg;
	if (lResult == 0)
	{
		CString strATR;
		Bytes2HexString(ATRBuff, ATRLength, strATR);
		strMsg.Format(_T("ATR: %s"), strATR);
	}

	OutputResult(lResult, GetFocus(), strMsg);
}


void CCiccWorkspace::OnBnClickedCiccDeactivate()
{
	LONG lResult = F3_RfcDeactivate(g_hReader);
	OutputResult(lResult, GetFocus(), NULL);
}


void CCiccWorkspace::OnBnClickedCiccStatus()
{
	BYTE bCardType;
	LONG lResult = F3_RfcGetStatus(g_hReader, &bCardType);

	CString strMsg;
	if (lResult == 0)
	{
		CString s;
		LoadString(IDS_ACTIVATED, s);
		switch (bCardType)
		{
		case RFCTYPE_MIFARE_S50: strMsg.Format(_T("Mifare S50 %s"), s); break;
		case RFCTYPE_MIFARE_S70: strMsg.Format(_T("Mifare S70 %s"), s); break;
		case RFCTYPE_MIFARE_UL: strMsg.Format(_T("Mifare Ultralight %s"), s); break;
		case RFCTYPE_TYPEA_CPU: strMsg.Format(_T("Type A CPU %s"), s); break;
		case RFCTYPE_TYPEB_CPU: strMsg.Format(_T("Type B CPU %s"), s); break;
		case RFCTYPE_UNKNOWN: LoadString(IDS_NO_RFC_ACTIVATED, strMsg); break;
		}

		Select(bCardType);
	}

	OutputResult(lResult, GetFocus(), strMsg);
}

void CCiccWorkspace::Select(BYTE bCardType)
{
	switch (bCardType)
	{
	case RFCTYPE_MIFARE_S50: m_tab.SelectPage(m_pS50Wnd); break;
	case RFCTYPE_MIFARE_S70: m_tab.SelectPage(m_pS70Wnd); break;
	case RFCTYPE_MIFARE_UL: m_tab.SelectPage(m_pUlWnd); break;
	case RFCTYPE_TYPEA_CPU: m_tab.SelectPage(m_pTypeACpuWnd); break;
	case RFCTYPE_TYPEB_CPU: m_tab.SelectPage(m_pTypeBCpuWnd); break;
	case RFCTYPE_RF_ISO15693: m_tab.SelectPage(m_pIsoWnd); break;
		
	}
}



void CCiccWorkspace::OnEnable(BOOL bEnable)
{
	CPanelBase::OnEnable(bEnable);

	GetDlgItem(IDC_CICC_ACTIVATE)->EnableWindow(bEnable);
	GetDlgItem(IDC_CICC_DEACTIVATE)->EnableWindow(bEnable);
	GetDlgItem(IDC_CICC_STATUS)->EnableWindow(bEnable);
	GetDlgItem(IDC_ISO14443_3_ACT)->EnableWindow(bEnable);
	GetDlgItem(IDC_ISO14443_4_ACT)->EnableWindow(bEnable);

	m_ActivationSequences.EnableWindow(bEnable);

	/*m_pS50Wnd->EnableWindow(bEnable);
	m_pS70Wnd->EnableWindow(bEnable);
	m_pUlWnd->EnableWindow(bEnable);
	m_pTypeACpuWnd->EnableWindow(bEnable);
	m_pTypeBCpuWnd->EnableWindow(bEnable);
	m_pIsoWnd->EnableWindow(bEnable);*/
}


void CCiccWorkspace::OnCbnSelchangeRfcActivationSequences()
{
	// TODO: ڴӿؼ֪ͨ
}


void CCiccWorkspace::OnBnClickedIso144433Act()
{
	BYTE ATRBuff[128];
	DWORD ATRLength = sizeof(ATRBuff);
	LONG lResult = F3_ISO14443_3_Activate(g_hReader, ATRBuff, &ATRLength);
	CString strMsg;
	if (lResult == 0)
	{
		CString strATR;
		Bytes2HexString(ATRBuff, ATRLength, strATR);
		strMsg.Format(_T("%s"), strATR);
	}

	OutputResult(lResult, GetFocus(), strMsg);
}


void CCiccWorkspace::OnBnClickedIso144434Act()
{
	BYTE ATRBuff[128];
	DWORD ATRLength = sizeof(ATRBuff);
	LONG lResult = F3_ISO14443_4_Activate(g_hReader, ATRBuff, &ATRLength);
	CString strMsg;
	if (lResult == 0)
	{
		CString strATR;
		Bytes2HexString(ATRBuff, ATRLength, strATR);
		strMsg.Format(_T("%s"), strATR);
	}

	OutputResult(lResult, GetFocus(), strMsg);
}
