// CmdListPanel.cpp : ʵļ
//

#include "stdafx.h"
#include "Public.h"
#include "XMLUtils.h"


COLORREF CMyListCtrl::OnGetCellTextColor(int nRow, int nColum)
{
	if (!m_bColor)
	{
		return CMFCListCtrl::OnGetCellTextColor(nRow, nColum);
	}

	return(nRow % 2) == 0 ? RGB(128, 37, 0) : RGB(0, 0, 0);
}

COLORREF CMyListCtrl::OnGetCellBkColor(int nRow, int nColum)
{
	if (!m_bColor)
	{
		return CMFCListCtrl::OnGetCellBkColor(nRow, nColum);
	}

	if (m_bMarkSortedColumn && nColum == m_iSortedColumn)
	{
		return(nRow % 2) == 0 ? RGB(233, 221, 229) : RGB(176, 218, 234);
	}

	return(nRow % 2) == 0 ? RGB(253, 241, 249) : RGB(196, 238, 254);
}

HFONT CMyListCtrl::OnGetCellFont(int nRow, int nColum, DWORD /*dwData* = 0*/)
{
	if (!m_bModifyFont)
	{
		return NULL;
	}

	if (nColum == 2 &&(nRow >= 4 && nRow <= 8))
	{
		return afxGlobalData.fontDefaultGUIBold;
	}

	return NULL;
}

int CMyListCtrl::OnCompareItems(LPARAM lParam1, LPARAM lParam2, int iColumn)
{
	CString strItem1 = GetItemText((int)(lParam1 < lParam2 ? lParam1 : lParam2), iColumn);
	CString strItem2 = GetItemText((int)(lParam1 < lParam2 ? lParam2 : lParam1), iColumn);

	if (iColumn == 0)
	{
		int nItem1 = _ttoi(strItem1);
		int nItem2 = _ttoi(strItem2);
		return(nItem1 < nItem2 ? -1 : 1);
	}
	else
	{
		int iSort = _tcsicmp(strItem1, strItem2);
		return(iSort);
	}
}



static CArray<LPCOMMANDINFO> gCommands;
static TCHAR gszFilePath[MAX_PATH];


static HRESULT LoadAllCommands(IXMLDOMDocument *pDoc)
{
	HRESULT hr = S_OK;
	IXMLDOMNode *pNodeItem = NULL;
	IXMLDOMNodeList *pNodeList = NULL;
	IXMLDOMNode *pAttrItem = NULL;
	IXMLDOMNamedNodeMap *pAttrList = NULL;
	BSTR nodeName;
	VARIANT nodeValue;
	long cItems;
	long cAttributes;
	LPCOMMANDINFO lpCmd;

	CHK_HR(SelectNodes(pDoc, L"Commands//Command", &pNodeList));
	CHK_HR(pNodeList->get_length(&cItems));
	for (long i = 0; i < cItems; i++) {
		CHK_HR(pNodeList->get_item(i, &pNodeItem));
		CHK_HR(pNodeItem->get_attributes(&pAttrList));
		CHK_HR(pAttrList->get_length(&cAttributes));

        lpCmd = new COMMANDINFO;
        if (lpCmd != NULL)
        {
		    for (long j = 0; j < cAttributes; j++) {
			    CHK_HR(pAttrList->get_item(j, &pAttrItem));
			    CHK_HR(pAttrItem->get_nodeName(&nodeName));
			    CHK_HR(pAttrItem->get_nodeValue(&nodeValue));

			    if (wcscmp(nodeName, L"Name") == 0)
                    lpCmd->strName = nodeValue.bstrVal;
			    else if (wcscmp(nodeName, L"DisplayData") == 0)
                    lpCmd->strDispData = nodeValue.bstrVal;
			    else if (wcscmp(nodeName, L"Data") ==0)
                    TUtils::HexStringToByteData(nodeValue.bstrVal, lpCmd->baData);
		    }
		    gCommands.Add(lpCmd);
        }
	}

CleanUp:
	return hr;
}


static BOOL LoadAllCommands(LPCTSTR pszFileName)
{
	HRESULT hr = S_OK;
	IXMLDOMDocument *pDoc = NULL;
	IXMLDOMElement *pRoot = NULL;
	IXMLDOMElement *pNode = NULL;
	IXMLDOMElement *pSubNode = NULL;
	IXMLDOMDocumentFragment *pDF = NULL;
	VARIANT_BOOL bIsSuccessful;

	VARIANT varFileName;
	VariantInit(&varFileName);

	CHK_HR(hr = CoInitialize(NULL));
	CHK_HR(CreateAndInitDOM(&pDoc));
	CHK_HR(VariantFromString(pszFileName, varFileName));
	CHK_HR(pDoc->load(varFileName, &bIsSuccessful));

	CHK_HR(LoadAllCommands(pDoc));

CleanUp:
	SAFE_RELEASE(pDoc);
	SAFE_RELEASE(pRoot);
	SAFE_RELEASE(pNode);
	SAFE_RELEASE(pDF);
	SAFE_RELEASE(pSubNode);
	CoUninitialize();
	return hr == S_OK;
}


static HRESULT SaveAllCommands(IXMLDOMDocument *pDoc, IXMLDOMNode *pParentNode)
{
	HRESULT hr = S_OK;
	IXMLDOMElement *pNode = NULL;
	IXMLDOMDocumentFragment *pDF = NULL;
	LPCOMMANDINFO lpCmd;
    CString strData;

	CHK_HR(pDoc->createDocumentFragment(&pDF));

	for (int i = 0; i < gCommands.GetCount(); i++) {
		lpCmd = gCommands[i];
		CHK_HR(CreateAndAddElementNode(pDoc, L"Command", L"\n\t", pDF, &pNode ));
        CHK_HR(CreateAndAddAttributeNode(pDoc, L"Name", lpCmd->strName, pNode));
        CHK_HR(CreateAndAddAttributeNode(pDoc, L"DisplayData", lpCmd->strDispData, pNode));
        
       TUtils:: ByteDataToHexString(lpCmd->baData, strData);
		CHK_HR(CreateAndAddAttributeNode(pDoc, L"Data", strData, pNode));
		SAFE_RELEASE(pNode);
	}

	CHK_HR(CreateAndAddTextNode(pDoc, L"\n", pDF));
	CHK_HR(AppendChildToParent(pDF, pParentNode));

CleanUp:
	SAFE_RELEASE(pNode);
	SAFE_RELEASE(pDF);
	return hr;
}


static BOOL SaveAllCommands(LPCTSTR pszFileName)
{
	HRESULT hr = S_OK;
	IXMLDOMDocument *pDoc = NULL;
	IXMLDOMElement *pRoot = NULL;
	IXMLDOMElement *pNode = NULL;
	IXMLDOMElement *pSubNode = NULL;
	IXMLDOMDocumentFragment *pDF = NULL;

	VARIANT varFileName;
	VariantInit(&varFileName);

	CHK_HR(hr = CoInitialize(NULL));
	CHK_HR(CreateAndInitDOM(&pDoc));
	CHK_HR(CreateAndAddPINode(pDoc, L"xml", L"version='1.0' encoding='gb2312'"));
	CHK_HR(CreateElement(pDoc, L"Commands", &pRoot));

	CHK_HR(SaveAllCommands(pDoc, pRoot));

	CHK_HR(AppendChildToParent(pRoot, pDoc));
	CHK_HR(VariantFromString(pszFileName, varFileName));
	CHK_HR(pDoc->save(varFileName));

CleanUp:
	SAFE_RELEASE(pDoc);
	SAFE_RELEASE(pRoot);
	SAFE_RELEASE(pNode);
	SAFE_RELEASE(pDF);
	SAFE_RELEASE(pSubNode);
	CoUninitialize();
	return hr == S_OK;
}



// CCmdListPanel Ի

IMPLEMENT_DYNAMIC(CCmdListPanel, UIPanel)

CCmdListPanel::CCmdListPanel(CWnd* pParent /*=NULL*/)
	: UIPanel(IDD_COMMAND_LIST_PANEL, pParent)
{
    m_nSel = -1;
    m_editDlg.SetEditCallback(CmdEditCallback, this);
    PathCombine(gszFilePath, TUtils::GetAppStartupPath(), _T("OtherCommands.xml"));
}

CCmdListPanel::~CCmdListPanel()
{
}

void CCmdListPanel::DoDataExchange(CDataExchange* pDX)
{
	UIPanel::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_CMD_LIST, m_wndCmdList);
}


BEGIN_MESSAGE_MAP(CCmdListPanel, UIPanel)
    ON_WM_SIZE()
    ON_WM_ENABLE()
    ON_BN_CLICKED(IDC_ADD_CMD, &CCmdListPanel::OnBnClickedAddCmd)
    ON_NOTIFY(NM_DBLCLK, IDC_CMD_LIST, &CCmdListPanel::OnNMDblclkCmdList)
END_MESSAGE_MAP()


// CCmdListPanel Ϣ

BOOL CCmdListPanel::OnInitDialog()
{
    UIPanel::OnInitDialog();

    m_wndCmdList.InsertColumn(0, _T(""), LVCFMT_LEFT, 200);
    m_wndCmdList.InsertColumn(1, _T(""), LVCFMT_LEFT, 200);
    ListAllCmds();

    return TRUE;  // return TRUE unless you set the focus to a control
    // 쳣: OCX ҳӦ FALSE
}


BOOL CCmdListPanel::PreTranslateMessage(MSG* pMsg)
{
    if (pMsg->message == WM_KEYDOWN)
    {
        UINT nCtrlID = GetWindowLong(pMsg->hwnd, GWL_ID);
        if (nCtrlID == m_wndCmdList.GetDlgCtrlID())
        {
            UINT nChar = pMsg->wParam;
            if (nChar == VK_DELETE)
            {
                OnDeleteSelCmd();
                return TRUE;
            }
            else if (nChar == VK_RETURN)
            {
                OnEditSelCmd();
                return TRUE;
            }
        }
    }

	return UIPanel::PreTranslateMessage(pMsg);
}


void CCmdListPanel::OnEnable(BOOL bEnable)
{
    GetDlgItem(IDC_ADD_CMD)->EnableWindow(bEnable);
}

void CCmdListPanel::OnSize(UINT nType, int cx, int cy)
{
	if (m_wndCmdList.GetSafeHwnd() == NULL)
		return;

    CRect rect;
    m_wndCmdList.GetWindowRect(&rect);
    ScreenToClient(&rect);
    m_wndCmdList.SetWindowPos(NULL, rect.left, rect.top, cx, cy, 
		SWP_NOACTIVATE | SWP_NOZORDER);
}


void CCmdListPanel::OnBnClickedAddCmd()
{
    m_editDlg.SetEditInfo(NULL);
    m_editDlg.DoModal();
}


void CCmdListPanel::OnNMDblclkCmdList(NMHDR *pNMHDR, LRESULT *pResult)
{
    ASSERT(m_fnDblClkCbk != NULL);

    LPNMITEMACTIVATE pNMItemActivate = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
    m_nSel = pNMItemActivate->iItem;
    if (m_nSel >= 0)
    {
        LPCOMMANDINFO pCmdInfo = (LPCOMMANDINFO)m_wndCmdList.GetItemData(m_nSel);
        m_fnDblClkCbk(m_pvParam, pCmdInfo);
    }
    *pResult = 0;
}


void CCmdListPanel::SetCDTMinLength(DWORD cbCDTMinLength)
{
    ASSERT(cbCDTMinLength >= 1);
    m_editDlg.SetCDTMinLength(cbCDTMinLength);
}



void CCmdListPanel::SetDblClkCallback(FNCMDLISTDBLCLKCALLBACK fnDblClkCbk, LPVOID pvParam)
{
    ASSERT(fnDblClkCbk != NULL);
    m_fnDblClkCbk = fnDblClkCbk;
    m_pvParam = pvParam;
}


void CCmdListPanel::ListAllCmds()
{
    gCommands.RemoveAll();
    if (!LoadAllCommands(gszFilePath))
        return;

    LPCOMMANDINFO lpCmd;
    for (int i = 0; i < gCommands.GetCount(); i++)
    {
        lpCmd = gCommands.GetAt(i);
        AddCmdToList(lpCmd);
    }
}


void CCmdListPanel::AddCmdToList(LPCOMMANDINFO lpCmdInfo)
{
    int iItem = m_wndCmdList.GetItemCount();
    m_wndCmdList.InsertItem(iItem, lpCmdInfo->strName);

    CString strData;
   TUtils:: ByteDataToHexString(lpCmdInfo->baData, strData);
    m_wndCmdList.SetItemText(iItem, 1, strData);

    m_wndCmdList.SetItemData(iItem, (DWORD_PTR)lpCmdInfo);
}


void CCmdListPanel::OnEditSelCmd()
{
    m_nSel = m_wndCmdList.GetSelectionMark();
    if (m_nSel < 0) return;

    LPCOMMANDINFO pCmdInfo = (LPCOMMANDINFO)m_wndCmdList.GetItemData(m_nSel);
    m_editDlg.SetEditInfo(pCmdInfo, FALSE);
    m_editDlg.DoModal();
}


void CCmdListPanel::OnDeleteSelCmd()
{
    m_nSel = m_wndCmdList.GetSelectionMark();
    if (m_nSel < 0) return;

    LPCOMMANDINFO pCmdInfo = (LPCOMMANDINFO)m_wndCmdList.GetItemData(m_nSel);
    for (int i = 0; i < gCommands.GetCount(); i++)
    {
        if (gCommands[i] == pCmdInfo)
        {
            gCommands.RemoveAt(i);
            break;
        }
    }
    
    m_wndCmdList.DeleteItem(m_nSel);
    SaveAllCommands(gszFilePath);
}


BOOL CCmdListPanel::CmdEditCallback(LPVOID pvParam, COMMANDINFO &CmdInfo, BOOL bAddNew)
{
    CCmdListPanel *pListPanel = (CCmdListPanel*)pvParam;
    if (bAddNew)
    {
        LPCOMMANDINFO pNewCmd = new COMMANDINFO;
        if (pNewCmd != NULL)
        {
            *pNewCmd = CmdInfo;
            pListPanel->AddCmdToList(pNewCmd);
            gCommands.Add(pNewCmd);
        }
    }
    else
    {
        pListPanel->m_wndCmdList.SetItemText(pListPanel->m_nSel, 0, CmdInfo.strName);
        pListPanel->m_wndCmdList.SetItemText(pListPanel->m_nSel, 1, CmdInfo.strDispData);
    }
    SaveAllCommands(gszFilePath);
    return TRUE;
}
