#include "StdAfx.h"
#include "Public.h"
#include "XMLUtils.h"


static HRESULT LoadAllStats(IXMLDOMDocument *pDoc, CArray<LPSTATUSINFO> &Stats)
{
	HRESULT hr = S_OK;
	IXMLDOMNode *pNodeItem = NULL;
	IXMLDOMNodeList *pNodeList = NULL;
	IXMLDOMNode *pAttrItem = NULL;
	IXMLDOMNamedNodeMap *pAttrList = NULL;
	BSTR nodeName;
	VARIANT nodeValue;
	long cItems;
	long cAttributes;
	LPSTATUSINFO lpStat;

	CHK_HR(SelectNodes(pDoc, L"Statuses//Status", &pNodeList));
	CHK_HR(pNodeList->get_length(&cItems));
	for (long i = 0; i < cItems; i++) {
		CHK_HR(pNodeList->get_item(i, &pNodeItem));
		CHK_HR(pNodeItem->get_attributes(&pAttrList));
		CHK_HR(pAttrList->get_length(&cAttributes));

        lpStat = new STATUSINFO;
        if (lpStat != NULL)
        {
		    for (long j = 0; j < cAttributes; j++) {
			    CHK_HR(pAttrList->get_item(j, &pAttrItem));
			    CHK_HR(pAttrItem->get_nodeName(&nodeName));
			    CHK_HR(pAttrItem->get_nodeValue(&nodeValue));

			    if (_wcsicmp(nodeName, L"Data") == 0)
                    lpStat->strData = nodeValue.bstrVal;
			    else if (_wcsicmp(nodeName, L"Meaning") == 0)
					lpStat->strMeaning = nodeValue.bstrVal;
		    }
		    Stats.Add(lpStat);
        }
	}

CleanUp:
	return hr;
}


static BOOL LoadAllStats(LPCTSTR pszFileName, CArray<LPSTATUSINFO> &Stats, STATDATATYPE &dataType)
{
	if (!PathFileExists(pszFileName))
		return FALSE;

	HRESULT hr = S_OK;
	IXMLDOMDocument *pDoc = NULL;
	IXMLDOMElement *pRoot = NULL;
	IXMLDOMNamedNodeMap *pAttrList = NULL;
	IXMLDOMNode *pAttrItem = NULL;
	VARIANT varFileName;
	VARIANT_BOOL bIsSuccessful;
	VARIANT nodeValue;
	BSTR nodeName;
	long cAttributes;
	
	VariantInit(&varFileName);

	CHK_HR(hr = CoInitialize(NULL));
	CHK_HR(CreateAndInitDOM(&pDoc));
	CHK_HR(VariantFromString(pszFileName, varFileName));
	CHK_HR(pDoc->load(varFileName, &bIsSuccessful));
	CHK_HR(pDoc->get_documentElement(&pRoot));
	if (pRoot == NULL) goto CleanUp;

	CHK_HR(pRoot->get_attributes(&pAttrList));
	CHK_HR(pAttrList->get_length(&cAttributes));
	for (long j = 0; j < cAttributes; j++)
	{
		CHK_HR(pAttrList->get_item(j, &pAttrItem));
		CHK_HR(pAttrItem->get_nodeName(&nodeName));
		CHK_HR(pAttrItem->get_nodeValue(&nodeValue));
		if (_wcsicmp(nodeName, L"DataType") == 0)
		{
			if (_wcsicmp(nodeValue.bstrVal, L"ASCII") == 0)
				dataType = SDT_ASCII;
			else
				dataType = SDT_HEXSTRING;
		}
	}
	CHK_HR(LoadAllStats(pDoc, Stats));

CleanUp:
	SAFE_RELEASE(pDoc);
	SAFE_RELEASE(pRoot);
	CoUninitialize();
	return hr == S_OK;
}



static HRESULT SaveAllStats(IXMLDOMDocument *pDoc, IXMLDOMNode *pParentNode, CArray<LPSTATUSINFO> &Stats)
{
	HRESULT hr = S_OK;
	IXMLDOMElement *pNode = NULL;
	IXMLDOMDocumentFragment *pDF = NULL;
	LPSTATUSINFO lpStat;

	CHK_HR(pDoc->createDocumentFragment(&pDF));

	for (int i = 0; i < Stats.GetCount(); i++) {
		lpStat = Stats[i];
		CHK_HR(CreateAndAddElementNode(pDoc, L"Status", L"\n\t", pDF, &pNode ));
		CHK_HR(CreateAndAddAttributeNode(pDoc, L"Data", lpStat->strData, pNode));
		CHK_HR(CreateAndAddAttributeNode(pDoc, L"Meaning", lpStat->strMeaning, pNode));
		SAFE_RELEASE(pNode);
	}

	CHK_HR(CreateAndAddTextNode(pDoc, L"\n", pDF));
	CHK_HR(AppendChildToParent(pDF, pParentNode));

CleanUp:
	SAFE_RELEASE(pNode);
	SAFE_RELEASE(pDF);
	return hr;
}


static BOOL SaveAllStats(LPCTSTR pszFileName, CArray<LPSTATUSINFO> &Stats, STATDATATYPE dataType)
{
	HRESULT hr = S_OK;
	IXMLDOMDocument *pDoc = NULL;
	IXMLDOMElement *pRoot = NULL;
	IXMLDOMElement *pNode = NULL;
	IXMLDOMElement *pSubNode = NULL;
	IXMLDOMDocumentFragment *pDF = NULL;

	VARIANT varFileName;
	VariantInit(&varFileName);

	CHK_HR(hr = CoInitialize(NULL));
	CHK_HR(CreateAndInitDOM(&pDoc));
	CHK_HR(CreateAndAddPINode(pDoc, L"xml", L"version='1.0' encoding='gb2312'"));
	CHK_HR(CreateElement(pDoc, L"Statuses", &pRoot));

	if (dataType == SDT_ASCII)
		CHK_HR(CreateAndAddAttributeNode(pDoc, L"DataType", L"ASCII", pRoot));
	else
		CHK_HR(CreateAndAddAttributeNode(pDoc, L"DataType", L"HexString", pRoot));

	CHK_HR(SaveAllStats(pDoc, pRoot, Stats));

	CHK_HR(AppendChildToParent(pRoot, pDoc));
	CHK_HR(VariantFromString(pszFileName, varFileName));
	CHK_HR(pDoc->save(varFileName));

CleanUp:
	SAFE_RELEASE(pDoc);
	SAFE_RELEASE(pRoot);
	SAFE_RELEASE(pNode);
	SAFE_RELEASE(pDF);
	SAFE_RELEASE(pSubNode);
	CoUninitialize();
	return hr == S_OK;
}


CStatusManager::CStatusManager()
	: m_PSDT(SDT_HEXSTRING)
	, m_NSDT(SDT_HEXSTRING)
{
	PathCombine(m_szPSFileName, TUtils::GetAppStartupPath(), _T("PositiveStatus.xml"));
	PathCombine(m_szNSFileName, TUtils::GetAppStartupPath(), _T("NegativeStatus.xml"));
}


CStatusManager::~CStatusManager()
{
	//SaveAllStats(m_szPSFileName, m_PositiveStats, m_PSDT);
	//SaveAllStats(m_szNSFileName, m_NegativeStats, m_NSDT);
	Release();
	
}


void CStatusManager::Initialize()
{
	LoadAllStats(m_szPSFileName, m_PositiveStats, m_PSDT);
	LoadAllStats(m_szNSFileName, m_NegativeStats, m_NSDT);
}


void CStatusManager::Release()
{
	for (int i = 0; i < m_PositiveStats.GetCount(); i++)
		delete m_PositiveStats[i];
	for (int i = 0; i < m_NegativeStats.GetCount(); i++)
		delete m_NegativeStats[i];

	m_PositiveStats.RemoveAll();
	m_NegativeStats.RemoveAll();
}


LPCTSTR CStatusManager::GetPositiveStatusMeaning(LPBYTE pbData, DWORD dwLength)
{
	return GetStatusMeaning(m_PositiveStats, pbData, dwLength, m_PSDT);
}


LPCTSTR CStatusManager::GetPositiveStatusMeaning(CByteArray &baData)
{
	return GetPositiveStatusMeaning(baData.GetData(), baData.GetCount());
}


LPCTSTR CStatusManager::GetNegativeStatusMeaning(LPBYTE pbData, DWORD dwLength)
{
	return GetStatusMeaning(m_NegativeStats, pbData, dwLength, m_NSDT);
}


LPCTSTR CStatusManager::GetNegativeStatusMeaning(CByteArray &baData)
{
	return GetNegativeStatusMeaning(baData.GetData(), baData.GetCount());
}


LPCTSTR CStatusManager::GetStatusMeaning(CArray<LPSTATUSINFO> &Stats, LPBYTE pbData, DWORD dwLength, STATDATATYPE dataType)
{
	if (Stats.GetCount() == 0)
		return _T("");

	if (pbData == NULL || dwLength == 0)
		return _T("");

	if (dataType == SDT_ASCII)
	{
		CStringA strDataA;
		for (int i = 0; i < Stats.GetCount(); i++)
		{
			strDataA = Stats[i]->strData;
			if (strncmp(strDataA.GetBuffer(0), (LPCSTR)pbData, dwLength) == 0)
				return Stats[i]->strMeaning;
		}
	}
	else
	{
		CByteArray baData;
		BOOL bEqual;
		for (int i = 0; i < Stats.GetCount(); i++)
		{
			TUtils::HexStringToByteData(Stats[i]->strData, baData);
			if (baData.GetCount() < dwLength)
				continue;

			bEqual = TRUE;
			for (int j = 0; j < dwLength; j++)
			{
				if (baData[j] != pbData[j])
				{
					bEqual = FALSE;
					break;
				}
			}

			if (bEqual)
				return Stats[i]->strMeaning;
		}
	}

	return _T("");
}