#include "StdAfx.h"
#include "Public.h"


extern AFX_EXTENSION_MODULE TCMNLIBDLL;


IMPLEMENT_DYNAMIC(UIInputBox, CEdit)

UIInputBox::UIInputBox()
{
}


UIInputBox::~UIInputBox()
{
}


BEGIN_MESSAGE_MAP(UIInputBox, CEdit)
	ON_WM_CHAR()
END_MESSAGE_MAP()


void UIInputBox::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	int bAccept = iscntrl(nChar);
    if (!bAccept)
    {
	    switch (m_intype)
	    {
	    case INTYPE_NUMBER:
		    bAccept = isdigit(nChar);
		    break;

	    case INTYPE_HEXNUMBER:
		    bAccept = isxdigit(nChar);
		    break;

	    case INTYPE_BYTEDATA:
		    bAccept = isxdigit(nChar) || isspace(nChar);
		    break;

	    case INTYPE_ASCIICHARS:
		    bAccept = isascii(nChar);
		    break;
	    }
    }

	if (bAccept)
		CEdit::OnChar(nChar, nRepCnt, nFlags);
}


void UIInputBox::SetInputType(INPUTTYPE intype)
{
	m_intype = intype;

	DWORD dwStyle = GetWindowLong(m_hWnd, GWL_STYLE);
	dwStyle &= ~ES_NUMBER;
    dwStyle &= ~WS_HSCROLL;
	if (intype == INTYPE_HEXNUMBER || intype == INTYPE_BYTEDATA)
		dwStyle |= ES_UPPERCASE;
	else
		dwStyle &= ~ES_UPPERCASE;

    if (intype == INTYPE_BYTEDATA)
        dwStyle |= ES_MULTILINE | ES_AUTOVSCROLL | WS_VSCROLL;
	SetWindowLong(m_hWnd, GWL_STYLE, dwStyle);
}


void UIInputBox::SetMinMaxValue(DWORD dwMin, DWORD dwMax)
{
	ASSERT(dwMax >= dwMin);

	m_dwMin = dwMin;
	m_dwMax = dwMax;
}


bool UIInputBox::GetValue(DWORD &dwValue)
{
	ASSERT(m_intype == INTYPE_NUMBER 
        || m_intype == INTYPE_HEXNUMBER);

	CString strText;
	GetWindowText(strText);
	
    strText.Trim();
    if (strText.GetLength() == 0)
    {
        ShowError(_T("δݡ"));
        return false;
    }

	if (m_intype == INTYPE_NUMBER)
		dwValue = _tcstol(strText, NULL, 10);
	else
		dwValue = _tcstol(strText, NULL, 16);

    if (dwValue < m_dwMin || dwValue > m_dwMax)
    {
        CString strError;
        strError.Format(_T("ֵΧ: %d ~ %d"), m_dwMin, m_dwMax);
        ShowError(strError);
        SetFocus();
        return false;
    }

    return true;
}


bool UIInputBox::GetData(CByteArray &ba)
{
    ASSERT(m_intype == INTYPE_BYTEDATA);

	CString strText;
	GetWindowText(strText);
	
    strText.Trim();
    if (strText.GetLength() == 0)
    {
        ShowError(_T("δݡ"));
        return false;
    }

    strText.Remove(_T(' '));
    strText.Remove(_T('\t'));
    strText.Remove(_T('\r'));
    strText.Remove(_T('\n'));
    TUtils::HexStringToByteData(strText, ba);
    return true;
}


void UIInputBox::ShowError(LPCTSTR pszError)
{
    CEdit::ShowBalloonTip(_T(""), pszError, TTI_ERROR);
    SetFocus();
}


void UIInputBox::ShowError(UINT nIDString)
{
    static TCHAR szError[256];
    LoadString(TCMNLIBDLL.hModule, nIDString, szError, 1024);
    ShowError(szError);
}
