#include "StdAfx.h"
#include "MSFlexGridExt.h"

#include <math.h>

// CMSFlexGridExt

IMPLEMENT_DYNAMIC(CMSFlexGridExt, CMSFlexGrid)

CMSFlexGridExt::CMSFlexGridExt()
{
	m_bReadOnly = false;
	m_lEditableCells = 0;
}


CMSFlexGridExt::~CMSFlexGridExt()
{
}


void CMSFlexGridExt::PreSubclassWindow()
{
         // ߿ĴС.
         put_Rows(0);
         put_Cols(0);
         m_lBorderWidth = get_CellLeft();
         m_lBorderHeight = get_CellTop();

         CDC* pDC = GetDC();
         m_nLogX = pDC->GetDeviceCaps(LOGPIXELSX);
         m_nLogY = pDC->GetDeviceCaps(LOGPIXELSY);
         ReleaseDC(pDC);

	m_EditWnd.Create(WS_CHILD|ES_MULTILINE|ES_WANTRETURN | ES_UPPERCASE,
            CRect(0,0,0,0), this, GetDlgCtrlID());
}

void CMSFlexGridExt::EditCell(short KeyAscii)
{
	if (m_bReadOnly) return;

	if ( m_lEditableCells > 0)
	{
		long iStartRow = get_FixedRows();
		long iStartCol = get_FixedCols();

		long nCells = ( get_Row() - iStartRow) * (get_Cols() - iStartCol) + (get_Col() - iStartCol);
		if (nCells >= m_lEditableCells)
			return;
	}

	m_EditWnd.SetLimitText(2);
	m_EditWnd.SetWindowText(get_Text());

	if (KeyAscii == VK_RETURN)
	{
		m_EditWnd.SetSel(0,-1);
	}
	else
	{
		TCHAR buf[] = { KeyAscii };
		m_EditWnd.SetSel(-1,-1);
		m_EditWnd.ReplaceSel(buf);
	}

	m_EditWnd.MoveWindow(((get_CellLeft() - m_lBorderWidth) *
		m_nLogX)/1440,
		((get_CellTop() - m_lBorderHeight) * m_nLogY)/1440,
		(get_CellWidth()* m_nLogX)/1440,
		(get_CellHeight()* m_nLogY)/1440, FALSE);

	m_EditWnd.ShowWindow(SW_SHOW);
	m_EditWnd.SetFocus();
}

void CMSFlexGridExt::EndEdit()
{
	if (m_bReadOnly) return;

	BOOL bVisible = ::GetWindowLong(m_EditWnd.GetSafeHwnd(), GWL_STYLE)
		& WS_VISIBLE;
	if (bVisible)
	{
		CString cStr;
		m_EditWnd.GetWindowText(cStr);
		put_Text(cStr);
		m_EditWnd.SetWindowText(_T(""));
		m_EditWnd.ShowWindow(SW_HIDE);
	}
}

void CMSFlexGridExt::SelectNextCell()
{
	long iColSel = get_ColSel();
	long iRowSel = get_RowSel();

	iColSel++;
	if (iColSel >= get_Cols())
	{
		iRowSel++;
		iColSel = get_FixedCols();
	}

	if (iRowSel >= get_Rows())
	{
		iRowSel = get_FixedRows();
	}
	put_Row(iRowSel);
	put_RowSel(iRowSel);
	put_Col(iColSel);
	put_ColSel(iColSel);

	OnEnterCell();
}


void CMSFlexGridExt::SetEditableCells(long lEditableCells)
{
	long iStartRow = get_FixedRows();
	long iStartCol = get_FixedCols();
	long nTotalRows = get_Rows();
	long nTotalCols = get_Cols();
	long iRow;
	long iCol;

	if (lEditableCells > m_lEditableCells && 
		m_lEditableCells > 0)
	{
		iRow = m_lEditableCells / (nTotalCols - iStartCol) + iStartRow;
		iCol = m_lEditableCells % (nTotalCols - iStartCol) + iStartCol;

		if (iRow <= nTotalRows && iCol < nTotalCols)
		{
			for (; iRow < nTotalRows; iRow++)
			{
				put_Row(iRow);
				for (; iCol < nTotalCols; iCol++)
				{
					put_Col(iCol);
					put_CellBackColor(RGB(255, 255, 255));
				}

				iCol = iStartCol;
			}
		}
	}

	m_lEditableCells = lEditableCells;
	iRow = m_lEditableCells / (nTotalCols - iStartCol) + iStartRow;
	iCol = m_lEditableCells % (nTotalCols - iStartCol) + iStartCol;

	if (iRow <= nTotalRows && iCol < nTotalCols)
	{
		for (; iRow < nTotalRows; iRow++)
		{
			put_Row(iRow);
			for (; iCol < nTotalCols; iCol++)
			{
				put_Col(iCol);
				put_CellBackColor(RGB(128, 128, 128));
			}

			iCol = iStartCol;
		}
	}
}

void CMSFlexGridExt::ResetContent()
{
	for (long iRow = get_FixedRows(); iRow < get_Rows(); iRow++)
	{
		for (long iCol = get_FixedCols(); iCol < get_Cols(); iCol++)
		{
			put_TextMatrix(iRow, iCol, _T(""));
		}
	}
}


BOOL CMSFlexGridExt::UpdateData(BOOL bSaveAndValidate)
{
	BOOL fRes = CMSFlexGrid::UpdateData(bSaveAndValidate);

	

	return fRes;
}

void CMSFlexGridExt::SelectCurrCell()
{
	OnEnterCell();
}

BEGIN_EVENTSINK_MAP(CMSFlexGridExt, CMSFlexGrid)
         ON_EVENT_REFLECT(CMSFlexGridExt, -603 /* KeyPress */, OnKeyPress,
                          VTS_PI2)
         ON_EVENT_REFLECT(CMSFlexGridExt, 72 /* LeaveCell */, OnLeaveCell,
                          VTS_NONE)
	ON_EVENT_REFLECT(CMSFlexGridExt, 71 /* EnterCell */, OnEnterCell,
			VTS_NONE)
END_EVENTSINK_MAP()


void CMSFlexGridExt::OnKeyPress(short FAR* KeyAscii)
{
	EditCell(*KeyAscii);
}

void CMSFlexGridExt::OnEnterCell()
{
	EditCell(VK_RETURN);
}

void CMSFlexGridExt::OnLeaveCell()
{
	EndEdit();
}


BEGIN_MESSAGE_MAP(CMSFlexGridExt, CMSFlexGrid)
	ON_WM_GETDLGCODE()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
END_MESSAGE_MAP()


UINT CMSFlexGridExt::OnGetDlgCode()
{
	return DLGC_WANTALLKEYS;
}


void CMSFlexGridExt::OnSetFocus(CWnd* pOldWnd)
{
	CMSFlexGrid::OnSetFocus(pOldWnd);

	OnLeaveCell();
}


void CMSFlexGridExt::OnKillFocus(CWnd* pNewWnd)
{
	CMSFlexGrid::OnKillFocus(pNewWnd);
}

//////////////////////////////////////////////////////////////////////////
// Helper methods

void CMSFlexGridExt::Init(long lTotalRows, long lTotalCols, long lFixedRows, long lFixedCols, long lCellWidth)
{
	put_Rows(lTotalRows);
	put_Cols(lTotalCols);
	put_FixedRows(lFixedRows);
	put_FixedCols(lFixedCols);

	for (long iCol = 0; iCol < lTotalCols; iCol++)
		put_ColWidth(iCol, lCellWidth);
}


void CMSFlexGridExt::InitRowHeader(long lStartVal)
{
	long lFixedRows = get_FixedRows();
	long lTotalRows = get_Rows();

	if (get_FixedCols() > 0 
		&& lTotalRows > 0)
	{
		CString strText;

		for (long iRow = lFixedRows; iRow < lTotalRows; iRow++)
		{
			strText.Format(_T("%X"), iRow - lFixedRows + lStartVal);
			put_TextMatrix(iRow, 0, strText);
		}
	}
}


void CMSFlexGridExt::InitColumnHeader(long lStartVal)
{
	long lFixedCols = get_FixedCols();
	long lTotalCols = get_Cols();

	if (get_FixedRows() > 0 
		&& lTotalCols > 0)
	{
		CString strText;

		for (long iCol = lFixedCols; iCol < lTotalCols; iCol++)
		{
			strText.Format(_T("%X"), iCol - lFixedCols + lStartVal);
			put_TextMatrix(0, iCol, strText);
		}
	}
}


bool CMSFlexGridExt::ValidateInput()
{
	EndEdit();

	long lFixedRows = get_FixedRows();
	long lFixedCols = get_FixedCols();
	long lTotalRows = get_Rows();
	long lTotalCols = get_Cols();
	long lEditableCells = GetEditableCells();

	CString strText;
	long nCount = 0;

	for (long iRow = lFixedRows; iRow < lTotalRows; iRow++)
	{
		for (long iCol = lFixedCols; iCol < lTotalCols; iCol++)
		{
			nCount++;

			strText = get_TextMatrix(iRow, iCol);
			if (strText.GetLength() == 0)
			{
				put_Row(iRow);
				put_RowSel(iRow);
				put_Col(iCol);
				put_ColSel(iCol);
				SetFocus();
				SelectCurrCell();
				return false;
			}

			if (lEditableCells > 0 &&
				nCount >= lEditableCells)
				goto L2;
		}
	}

L2:
	return true;
}


__forceinline char HexChar(BYTE digit)
{
	if (digit <= 9)
		return '0' + digit;
	else
		return 'A' + (digit - 10);
}

void Bytes2HexString(PBYTE pbBuffer, DWORD cbBuffer, CString& str)
{
	str.Empty();

	for (DWORD i = 0; i < cbBuffer; i++)
	{
		str.AppendChar(HexChar(pbBuffer[i] / 16));
		str.AppendChar(HexChar(pbBuffer[i] % 16));
	}
}


void CMSFlexGridExt::Fill(PBYTE pbBuffer, DWORD dwLength, bool bAdjustCells)
{
	EndEdit();

	long lFixedRows = get_FixedRows();
	long lFixedCols = get_FixedCols();
	long lTotalRows = get_Rows();
	long lTotalCols = get_Cols();

	if (bAdjustCells)
	{
		long lMinRows = (long) ceil(dwLength / (float)(lTotalCols - lFixedCols));
		lTotalRows = lMinRows + lFixedRows;
		put_Rows(lTotalRows);
	}

	DWORD nCount = 0;
	bool bContinue = true;
	CString strText;

	for (long iRow = lFixedRows; iRow < lTotalRows && bContinue ; iRow++)
	{
		for (long iCol = lFixedCols; iCol < lTotalCols && bContinue; iCol++)
		{
			Bytes2HexString(pbBuffer++, 1, strText);
			put_TextMatrix(iRow, iCol, strText);

			nCount++;
			if (nCount >= dwLength)
				bContinue = false;
		}
	}

	if (! IsReadOnly())
		SetEditableCells(dwLength);
}


DWORD CMSFlexGridExt::GetContent(PBYTE pbBuffer, DWORD cchBufferMax)
{
	EndEdit();

	long lFixedRows = get_FixedRows();
	long lFixedCols = get_FixedCols();
	long lTotalRows = get_Rows();
	long lTotalCols = get_Cols();


	DWORD dwCount = 0;
	CString strText;
	BYTE cVal;
	
	long iRow = lFixedRows;
	long iCol = lFixedCols;

	for (long i = 0; i < (long) cchBufferMax; i++)
	{
		strText = get_TextMatrix(iRow, iCol);
		iCol++;

		if (strText.GetLength() > 0)
		{
			cVal = (BYTE) _tcstol(strText, NULL, 16);
			pbBuffer[i] = cVal;
			dwCount++;
		}
		else
			pbBuffer[i] = 0xff;

		if (iCol % lTotalCols == 0)
		{
			iRow++;
			if (iRow >= lTotalRows)
				break;

			iCol = lFixedCols;
		}
	}

	return dwCount;
}