// NumberBox.cpp : ʵļ
//

#include "stdafx.h"
#include "resource.h"
#include "NumericEdit.h"

extern AFX_EXTENSION_MODULE UIEXTENDDLL;

void LoadStringW(UINT nID, CString& str)
{
	static TCHAR szBuffer[1024];
	::LoadString(UIEXTENDDLL.hModule, nID, szBuffer, 1024);
	str = szBuffer;
}

// CNumericEdit

IMPLEMENT_DYNAMIC(CNumericEdit, CEdit)

CNumericEdit::CNumericEdit()
{
	m_bAllowHexInput = FALSE;
	m_iMin = 0;
	m_iMax = UINT_MAX;
	m_iValue = 0;
}

CNumericEdit::~CNumericEdit()
{
}


BEGIN_MESSAGE_MAP(CNumericEdit, CEdit)
	ON_WM_CHAR()
END_MESSAGE_MAP()



// CNumericEdit Ϣ




BOOL CNumericEdit::Create(DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID)
{
	dwStyle &= ~ES_NUMBER;
	dwStyle |= ES_UPPERCASE;

	BOOL fRes = CEdit::Create(dwStyle, rect, pParentWnd, nID);
	if (fRes)
	{
		SetValue(m_iValue);
	}

	return fRes;
}


void CNumericEdit::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if ( iscntrl(nChar))
	{
		CEdit::OnChar(nChar, nRepCnt, nFlags);
		return;
	}

	bool fValid = true;
	UINT nErrID = 0;

	 if (m_bAllowHexInput)
	{
		if (! isxdigit(nChar))
		{
			fValid = FALSE;
			nErrID = IDS_ACCEPT_HEXADECIMAL_DIGITS_ONLY;
		}
	}
	 else if (! isdigit(nChar))
	 {
		 fValid = FALSE;
		nErrID = IDS_ACCEPT_DECIMAL_DIGITS_ONLY;
	 }

	if (! fValid)
	{
		CString strTitle;
		LoadString(IDS_UNACCEPTABLE_CHARACTER, strTitle);

		CString strText;
		LoadString(nErrID, strText);

		CEdit::ShowBalloonTip(strTitle, strText, TTI_ERROR);
	}
	else
	{
		CEdit::OnChar(nChar, nRepCnt, nFlags);
	}
}

void	CNumericEdit::AllowHexInput()
{
	m_bAllowHexInput = TRUE;
}

void	CNumericEdit::SetRange(UINT iMin, UINT iMax)
{
	m_iMin = iMin;
	m_iMax = iMax;
}

bool	CNumericEdit::IsValidInput()
{
	bool	fValid = true;

	CString strText;
	GetWindowText(strText);

	if (strText.GetLength() == 0)
	{
		if (m_bAllowHexInput)
		{
			LoadString(IDS_EXPECT_INPUT_HEXADECIMAL_DIGIT, strText);
			fValid = false;
		}
		else
		{
			LoadString(IDS_EXPECT_INPUT_DECIMAL_DIGIT, strText);
			fValid = false;
		}
	}
	else
	{
		UINT iValue;
		if ( m_bAllowHexInput)
			iValue = (UINT) _tcstol(strText, NULL, 16);
		else
			iValue = (UINT) _tstoi(strText);

		if (iValue > m_iMax || iValue < m_iMin)
		{
			CString strFormat;
			LoadString(IDS_VALUE_OUT_OF_RANGE, strFormat);
			strText.Format(strFormat, m_iMin, m_iMax);
			fValid = false;
		}
	}

	if (! fValid)
	{
		CString strTitle;
		LoadString(IDS_INVALID_INPUT, strTitle);

		CEdit::ShowBalloonTip(strTitle, strText, TTI_ERROR);
	}
	
	return fValid;
}

void CNumericEdit::SetValue(UINT iValue)
{
	m_iValue = iValue;
	if (IsWindow(m_hWnd))
	{
		CString str;
		if (m_bAllowHexInput)
			str.Format(_T("%x"), m_iValue);
		else
			str.Format(_T("%d"), m_iValue);

		SetWindowText(str);
	}
}

UINT CNumericEdit::GetValue()
{
	CString str;
	GetWindowText(str);

	if (m_bAllowHexInput)
		return (UINT) _tcstol(str, NULL, 16);
	else
		return (UINT) _tstoi(str);
}