#include "stdafx.h"
#include "TabCtrlExt.h"

#define PADDING  2

// CTabCtrlExt

IMPLEMENT_DYNAMIC(CTabCtrlExt, CTabCtrl)

CTabCtrlExt::CTabCtrlExt()
: m_iCurSel(0)
{

}

CTabCtrlExt::~CTabCtrlExt()
{
	m_arPages.RemoveAll();
	m_arItemData.RemoveAll();
}


BEGIN_MESSAGE_MAP(CTabCtrlExt, CTabCtrl)
	ON_NOTIFY_REFLECT_EX(TCN_SELCHANGE, &CTabCtrlExt::OnTcnSelchange)
	ON_WM_ENABLE()
	ON_WM_SIZE()
	ON_WM_CREATE()
END_MESSAGE_MAP()


// CTabCtrlExt Ϣ

BOOL CTabCtrlExt::OnTcnSelchange(NMHDR *pNMHDR, LRESULT *pResult)
{
	if (m_arPages.GetCount() == 0)
		return FALSE;

	CWnd *pPage = m_arPages[m_iCurSel];
	if (pPage != NULL)
		pPage->ShowWindow(SW_HIDE);
	
	m_iCurSel = GetCurSel();

	pPage = m_arPages[m_iCurSel];
	if (pPage != NULL)
	{
		pPage->ShowWindow(SW_SHOW);
		pPage->SetFocus();
	}

	*pResult = 0;

	return FALSE;
}



void CTabCtrlExt::OnSize(UINT nType, int cx, int cy)
{
	CTabCtrl::OnSize(nType, cx, cy);

	CWnd *pPage;
	int nCount = GetItemCount();

	for (int i = 0; i < nCount; i++)
	{
		pPage = m_arPages[i];
		if (pPage != NULL)
			PositionPage(i, pPage, i == m_iCurSel);
	}
}


void CTabCtrlExt::OnEnable(BOOL bEnable)
{
	CTabCtrl::OnEnable(bEnable);

	CWnd *pPage;
	int nCount = GetItemCount();

	for (int i = 0; i < nCount; i++)
	{
		pPage = m_arPages[i];
		if (pPage != NULL)
			pPage->EnableWindow(bEnable);
	}
}


LONG CTabCtrlExt::AddPage(LPCTSTR lpszItem, CWnd* pPage)
{
	ASSERT_VALID(pPage);

	pPage->SetWindowText(lpszItem);
	LONG iIndex = CTabCtrl::InsertItem(GetItemCount(), lpszItem);
	if (iIndex >= 0)
	{
		pPage->SetParent(this);
		m_arPages.SetAtGrow(iIndex, pPage);
		PositionPage(iIndex, pPage, iIndex == m_iCurSel);
	}

	return iIndex;
}

void CTabCtrlExt::SelectPage(CWnd *pPage)
{
	int nCount = GetItemCount();
	for (int i = 0; i < nCount; i++)
	{
		if (m_arPages[i] == pPage)
		{
			LRESULT result;
			SetCurSel(i);
			OnTcnSelchange(NULL, &result);
			break;
		}
	}
}

void CTabCtrlExt::SetItemData(int nItem, DWORD_PTR dwItemData)
{
	m_arItemData.SetAtGrow(nItem, dwItemData);
}


DWORD_PTR CTabCtrlExt::GetItemData(int nItem)
{
	return m_arItemData[nItem];
}

void CTabCtrlExt::PositionPage(int nItem, CWnd* pPage, BOOL bShow)
{
	int nBorderHeight = GetSystemMetrics(SM_CYFRAME);
	int nBorderWidth = GetSystemMetrics(SM_CXFRAME);

	CRect itemRect;
	GetItemRect(nItem, &itemRect);

	CRect clientRect;
	GetClientRect(&clientRect);

	clientRect.left += nBorderWidth + PADDING;
	clientRect.right -= nBorderWidth  + PADDING;
	clientRect.top += itemRect.Height() + nBorderHeight + PADDING;
	clientRect.bottom -= nBorderHeight + PADDING;

	pPage->SetWindowPos(NULL, 
		clientRect.left, 
		clientRect.top, 
		clientRect.Width(), 
		clientRect.Height(),
		bShow ? SWP_SHOWWINDOW : SWP_HIDEWINDOW);
}

void CTabCtrlExt::PreSubclassWindow()
{
	// TODO: ڴרô/û

	CTabCtrl::PreSubclassWindow();

	CFont *pFont = GetFont();
	if (pFont != NULL)
	{
		LOGFONT lfont;
		pFont->GetLogFont(&lfont);
		lfont.lfWeight = FW_BOLD;
		m_BoldFont.CreateFontIndirect(&lfont);
		SetFont(&m_BoldFont);
	}
}


int CTabCtrlExt::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CTabCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;

	CWnd *pParent = this->GetParent();
	if (pParent != NULL)
	{
		SetFont(pParent->GetFont());
	}
	return 0;
}
