﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MTKF31
{
    public partial class Form1 : Form
    {
        MTKF31DeviceObj _readerObj = new MTKF31DeviceObj();

        public Form1()
        {
            InitializeComponent();
        }


        private void Form1_Load(object sender, EventArgs e)
        {
            String[] portNames = SerialPort.GetPortNames();
            if (portNames.Length > 0)
            {

                cboF31Ports.Items.AddRange(portNames);
                cboF31Ports.SelectedIndex = 0;
            }

            UpdateUIState(false);

            txtF31APDUCommand.Text = "A0A40000023F00";
        }



        private void btnF31Connect_Click(object sender, EventArgs e)
        {
            string portName = cboF31Ports.SelectedItem.ToString();
            int ret = _readerObj.Connect(portName, 0);
            if (ret != 0)
                ReportF31Error(ret);
            else
                UpdateUIState(true);
        }


        private void btnF31Disconnect_Click(object sender, EventArgs e)
        {
            _readerObj.Disconnect();
            UpdateUIState(false);
        }



        private void btnF31ResetNoAction_Click(object sender, EventArgs e)
        {
            string fwver = null;
            int ret = _readerObj.Reset(MTKF31ResetAction.NoAction, ref fwver);
            if (ret != 0)
                ReportF31Error(ret);
            else
                MessageBox.Show(fwver, "Firmware Version", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void btnF31ResetEject_Click(object sender, EventArgs e)
        {
            string fwver = null;
            int ret = _readerObj.Reset(MTKF31ResetAction.Eject, ref fwver);
            if (ret != 0)
                ReportF31Error(ret);
            else
                MessageBox.Show(fwver, "Firmware Version", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void btnF31ResetRetain_Click(object sender, EventArgs e)
        {
            string fwver = null;
            int ret;

            ret = _readerObj.Reset(MTKF31ResetAction.Retain, ref fwver);
            if (ret != 0)
            {
                ReportF31Error(ret);
                return;
            }

            MessageBox.Show(fwver, "Firmware Version", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void btnF31Status_Click(object sender, EventArgs e)
        {
            MTKF31Status status = MTKF31Status.Unknown;
            int ret = _readerObj.GetStatus(ref status);
            if (ret != 0)
            {
                ReportF31Error(ret);
                return;
            }

            string msg;
            switch (status)
            {
                case MTKF31Status.CardAtExitSlot:
                    msg = "Card at exit slot";
                    break;
                case MTKF31Status.CardAtReadPos:
                    msg = "Card at read pos";
                    break;
                case MTKF31Status.NoCardPresent:
                    msg = "No card present";
                    break;
                default:
                    msg = "Unknown";
                    break;
            }

            MessageBox.Show(msg, "Status", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void btnF31Dispense_Click(object sender, EventArgs e)
        {
            int ret = _readerObj.Dispense();
            if (ret != 0)
            {
                ReportF31Error(ret);
            }
        }

        private void btnF31Eject_Click(object sender, EventArgs e)
        {
            int ret = _readerObj.Eject();
            if (ret != 0)
            {
                ReportF31Error(ret);
            }
        }

        private void btnF31Retain_Click(object sender, EventArgs e)
        {
            int ret;

            ret = _readerObj.Retain();
            if (ret != 0)
            {
                ReportF31Error(ret);
                return;
            }

        }



        private void btnF31MoveToICCPos_Click(object sender, EventArgs e)
        {
            int ret = _readerObj.MoveCardToICCPos();
            if (ret != 0)
            {
                ReportF31Error(ret);
            }
        }

        private void btnF31MoveToRFPos_Click(object sender, EventArgs e)
        {
            int ret = _readerObj.MoveCardToRFPos();
            if (ret != 0)
            {
                ReportF31Error(ret);
            }
        }

        private void btnF31ICCPowerOn_Click(object sender, EventArgs e)
        {
            bool isT0 = false;
            byte[] atr = null;
            int ret = _readerObj.ICCPowerOn(ref isT0, ref atr);
            if (ret != 0)
            {
                ReportF31Error(ret);
                return;
            }

            string text = BitConverter.ToString(atr);
            txtF31ATR.Text = text.Replace("-", "");
        }

        private void btnF31ICCPowerOff_Click(object sender, EventArgs e)
        {
            int ret = _readerObj.ICCPowerOff();
            if (ret != 0)
            {
                ReportF31Error(ret);
            }
        }

        private void btnF31ICCTransmit_Click(object sender, EventArgs e)
        {
            string text = txtF31APDUCommand.Text.Replace(" ", "");
            byte[] senddata = StringToByteArray(text);
            byte[] recvdata = null;
            int ret = _readerObj.ICCTransmitT0(senddata, ref recvdata);
            if (ret != 0)
            {
                ReportF31Error(ret);
                return;
            }

            text = BitConverter.ToString(recvdata);
            txtF31APDUResponse.Text = text.Replace("-", "");
        }


        void UpdateUIState(bool connected)
        {
            btnF31Connect.Enabled = !connected;
            btnF31Disconnect.Enabled = connected;
            btnF31ResetNoAction.Enabled = connected;
            btnF31ResetEject.Enabled = connected;
            btnF31ResetRetain.Enabled = connected;
            btnF31Status.Enabled = connected;
            btnF31Dispense.Enabled = connected;
            btnF31Eject.Enabled = connected;
            btnF31Retain.Enabled = connected;
            btnF31ICCPowerOff.Enabled = connected;
            btnF31ICCPowerOn.Enabled = connected;
            btnF31ICCTransmit.Enabled = connected;
            btnMoveCardToICCPos.Enabled = connected;
            btnF31MoveToRFPos.Enabled = connected;
            btnF31RFActivate.Enabled = connected;
            btnF31RFTransmit.Enabled = connected;
            btnF31RFDeactivate.Enabled = connected;
        }


        void ReportF31Error(int ret)
        {
            string msg = MTKF31DeviceObj.err2str(ret);
            MessageBox.Show(msg, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }


        public byte[] StringToByteArray(string hex)
        {
            return Enumerable.Range(0, hex.Length)
                             .Where(x => x % 2 == 0)
                             .Select(x => Convert.ToByte(hex.Substring(x, 2), 16))
                             .ToArray();
        }

        private void btnF31RFActivate_Click(object sender, EventArgs e)
        {
            bool isTypeA = false;
            byte[] atr = null;
            int ret = _readerObj.RFActivate(ref isTypeA, ref atr);
            if (ret != 0)
            {
                ReportF31Error(ret);
                return;
            }

            string text = BitConverter.ToString(atr);
            txtF31ATR.Text = text.Replace("-", "");
        }

        private void btnF31RFTransmit_Click(object sender, EventArgs e)
        {
            string text = txtF31APDUCommand.Text.Replace(" ", "");
            byte[] senddata = StringToByteArray(text);
            byte[] recvdata = null;
            int ret = _readerObj.RFTransmitTypeA(senddata, ref recvdata);
            if (ret != 0)
            {
                ReportF31Error(ret);
                return;
            }

            text = BitConverter.ToString(recvdata);
            txtF31APDUResponse.Text = text.Replace("-", "");
        }

        private void btnF31RFDeactivate_Click(object sender, EventArgs e)
        {
            int ret = _readerObj.RFDeactivate();
            if (ret != 0)
            {
                ReportF31Error(ret);
            }
        }

        private void btnF31EnableInsertion_Click(object sender, EventArgs e)
        {
            int ret = _readerObj.AllowInsertion(true);
            if (ret != 0)
            {
                ReportF31Error(ret);
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            int ret = _readerObj.AllowInsertion(false);
            if (ret != 0)
            {
                ReportF31Error(ret);
            }
        }
    }
}
