﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO.Ports;

namespace MTKF31
{
    class USARTStream
    {
        SerialPort _port = new SerialPort();
        List<byte> _recvBuf = new List<byte>();
        int _readPos = 0;

        public void Open(string portName, int baudRate)
        {
            _port.PortName = portName;
            _port.BaudRate = 9600;
            _port.Parity = Parity.None;
            _port.StopBits = StopBits.One;
            _port.Open();
            _port.DataReceived += DataReceivedEventHandler;
            _readPos = 0;
        }

        public void Close()
        {
            if (_port.IsOpen)
                _port.Close();
        }


        public bool IsOpen
        {
            get { return _port.IsOpen;  }
        }


        public void WaitBytesAvailable(int n, int timeout)
        {
            Stopwatch sw = new Stopwatch();
            sw.Start();

            while (_recvBuf.Count - _readPos < n)
            {
                if (sw.ElapsedMilliseconds > timeout)
                {
                    sw.Stop();
                    throw new TimeoutException();
                }
            }

            sw.Stop();
        }


        public void Write(byte[] buffer, int offset, int count)
        {
            _port.Write(buffer, offset, count);
        }


        public void Read(byte[] buffer, int offset, int count)
        {
            _recvBuf.CopyTo(_readPos, buffer, offset, count);
            _readPos += count;
        }


        public void DiscardIOBuffers()
        {
            _port.DiscardInBuffer();
            _port.DiscardOutBuffer();
            _recvBuf.Clear();
            _readPos = 0;
        }

        private void DataReceivedEventHandler(object sender, SerialDataReceivedEventArgs e)
        {
            SerialPort port = (SerialPort)sender;
            int n = port.BytesToRead;
            byte[] buf = new byte[n];
            port.Read(buf, 0, n);
            _recvBuf.AddRange(buf);
        }
    }
}
