#include "stdafx.h"
#include "F3API.h"
#include <atlstr.h>

#define TAG_READ	"Read "
#define TAG_WRITE	"Write"

//
// VID  PID
//
#define VID		0x4144 
#define PID		0x5248

//////////////////////////////////////////////
// CPortBase
//
CPortBase::CPortBase()
: m_hFile(INVALID_HANDLE_VALUE)
{
    RtlZeroMemory(&m_oRead, sizeof(m_oRead));
    RtlZeroMemory(&m_oWrite, sizeof(m_oWrite));
}

CPortBase::~CPortBase()
{
    Release();
}

/**************************************************************
*˿ڳʼ 
*˵ 
***************************************************************/
int CPortBase::Initialize(char *pDevPath, DWORD dwBps,CLogger *pLogger)
{
    int ret;
    ret = InitPort(pDevPath,dwBps);
    if (ret != 0) return ret;

    ret = InitEvents();
    if (ret != 0)
    {
        ReleasePort();
        return ret;
    }
    m_pLogger = pLogger;
    return 0;
}

void CPortBase::Release()
{
    ReleaseEvents();
    ReleasePort();
}


void CPortBase::ReleasePort()
{
    if (m_hFile != INVALID_HANDLE_VALUE)
    {
        CloseHandle(m_hFile);
        m_hFile = INVALID_HANDLE_VALUE;
    }
}

int CPortBase::InitEvents()
{
    m_oRead.hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if (m_oRead.hEvent == NULL)
        return F3_E_UNKNOWN_ERROR;

    m_oWrite.hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if (m_oWrite.hEvent == NULL)
        return F3_E_UNKNOWN_ERROR;

    return 0;
}


void CPortBase::ReleaseEvents()
{
    if (m_oRead.hEvent != NULL)
    {
        CloseHandle(m_oRead.hEvent);
        m_oRead.hEvent = NULL;
    }

    if (m_oWrite.hEvent != NULL)
    {
        CloseHandle(m_oWrite.hEvent);
        m_oWrite.hEvent = NULL;
    }
}


//////////////////////////////////////////////
// CSerialPort
//


CSerialPort::CSerialPort()
: CPortBase()
{
}

CSerialPort::~CSerialPort( )
{
}

int CSerialPort::ReadData(LPBYTE pbBuff, DWORD dwBytesToRead, LPDWORD pdwBytesRead)
{
    DWORD dwBytesRead;
    DWORD dwError;

    if (!ReadFile(m_hFile, pbBuff, dwBytesToRead, &dwBytesRead, &m_oRead))
    {
        dwError = GetLastError();
        if (dwError == ERROR_IO_PENDING)
        {
            switch (WaitForSingleObject(m_oRead.hEvent, INFINITE))
            {
            case WAIT_OBJECT_0:
                if (GetOverlappedResult(m_hFile, &m_oRead, &dwBytesRead, FALSE))
                {
                    if (dwBytesRead != dwBytesToRead)
                        return  F3_E_COMM_ERROR;
                    else
                        break;
                } else
                {
                    return F3_E_UNKNOWN_ERROR;
                }

            case WAIT_TIMEOUT:
                return F3_E_COMM_TIMEOUT;

            default:
                return F3_E_UNKNOWN_ERROR;
            }
        } else
        {
            return F3_E_UNKNOWN_ERROR;
        }
    }else
    {
       if (dwBytesRead != dwBytesToRead)
            return F3_E_UNKNOWN_ERROR;
    }

    if (pdwBytesRead != NULL)
        *pdwBytesRead = dwBytesRead;

    m_pLogger->Write(TAG_READ, pbBuff, dwBytesRead);
    return 0;
}


int CSerialPort::WriteData(LPBYTE pbBuff, DWORD dwBytesToWrite)
{
    DWORD dwBytesWritten;
    DWORD dwError;

    if (!WriteFile(m_hFile, pbBuff, dwBytesToWrite, &dwBytesWritten, &m_oWrite))
    {
        dwError = GetLastError();
        if (dwError == ERROR_IO_PENDING)
        {
            switch (WaitForSingleObject(m_oWrite.hEvent, INFINITE))
            {
            case WAIT_OBJECT_0:
                if (GetOverlappedResult(m_hFile, &m_oWrite, &dwBytesWritten, FALSE))
                {
                    if (dwBytesWritten != dwBytesToWrite)
                        return F3_E_UNKNOWN_ERROR;
                    else
                        break;
                } else
                {
                    return F3_E_UNKNOWN_ERROR;
                }

            case WAIT_TIMEOUT:
                return F3_E_COMM_TIMEOUT;

            default:
                return F3_E_UNKNOWN_ERROR;
            }
        } else
        {
            return F3_E_UNKNOWN_ERROR;
        }
    }

    m_pLogger->Write(TAG_WRITE, pbBuff, dwBytesToWrite);
    return 0;
}


int CSerialPort::WaitDataAvailable(DWORD dwTimeOut)
{
    COMSTAT CommStat;
    DWORD dwCurrentTime;
    DWORD dwStartTime;
    DWORD dwElapsedTime;
    DWORD dwErrors;

    dwStartTime = GetTickCount();
    dwElapsedTime = 0;
	MSG msg;
	CString str;
	int nCount = 0;

    while (TRUE)
    {
        if (!ClearCommError(m_hFile, &dwErrors, &CommStat))
            return F3_E_UNKNOWN_ERROR;

		//str.Format(_T("cbInQue = %d nCount = %d "),CommStat.cbInQue,nCount++);
		//MessageBox(NULL,str,0,0);
        if (CommStat.cbInQue > 0)// յֽ
            return 0;

        dwCurrentTime = GetTickCount();
        if ( dwCurrentTime >= dwStartTime )
            dwElapsedTime += dwCurrentTime - dwStartTime;
        else
            dwElapsedTime += INFINITE - dwStartTime + dwCurrentTime;
		//str.Format(_T("dwElapsedTime = %d  dwTimeOut = %d "),dwElapsedTime,dwTimeOut);
		//MessageBox(NULL,str,0,0);
        if (dwElapsedTime >= dwTimeOut)
            return F3_E_COMM_TIMEOUT;

        dwStartTime = dwCurrentTime;
        Sleep(50);
		if(::PeekMessage(&msg,NULL,0,0,PM_NOREMOVE));
    }
}


int CSerialPort::ResetInput()
{
    if (!PurgeComm(m_hFile, PURGE_FLAG))
        return F3_E_UNKNOWN_ERROR;
    return 0;
}
/**************************************************************
*pDevPath:·
***************************************************************/
int CSerialPort::InitPort(char *pDevPath,DWORD dwBps)
//int CSerialPort::InitPort(DWORD dwPortNumber)
{
    if (m_hFile == INVALID_HANDLE_VALUE)
    {
//      char szPortName[32];
//      sprintf_s(szPortName,
//                sizeof(szPortName),
//                "\\\\.\\COM%d",
//                dwPortNumber);

        m_hFile = CreateFileA(pDevPath, 
                              GENERIC_READ | GENERIC_WRITE, 0, NULL,
                              OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
        if (m_hFile == INVALID_HANDLE_VALUE)
        {
            switch (GetLastError())
            {
            case ERROR_FILE_NOT_FOUND:
            case ERROR_ACCESS_DENIED:
                return F3_E_PORT_UNAVAILABLE;
            default:
                return F3_E_UNKNOWN_ERROR;
            }
        }
    }

    DCB dcb;
    if (!GetCommState(m_hFile, &dcb))
        return F3_E_UNKNOWN_ERROR;

    dcb.BaudRate = dwBps;
    dcb.Parity = NOPARITY;
    dcb.ByteSize = DATABITS_8;
    dcb.StopBits = ONESTOPBIT;
    if (!SetCommState(m_hFile, &dcb))
        return F3_E_UNKNOWN_ERROR;

    if (!SetupComm(m_hFile, 4096, 4096))
        return F3_E_UNKNOWN_ERROR;

    COMMTIMEOUTS timeouts;
    timeouts.ReadIntervalTimeout = 50;
    timeouts.ReadTotalTimeoutMultiplier = 50;
    timeouts.ReadTotalTimeoutConstant = 100;
    timeouts.WriteTotalTimeoutMultiplier = 50;
    timeouts.WriteTotalTimeoutConstant = 100;
    if (!SetCommTimeouts(m_hFile, &timeouts))
        return F3_E_UNKNOWN_ERROR;

    return 0;
}



//////////////////////////////////////////////
// CUsbPort
//

CUsbPort::CUsbPort()
: CPortBase()
,m_pbInRptBuff(NULL)
,m_pbOutRptBuff(NULL)
{
}


CUsbPort::~CUsbPort( )
{
    if (m_pbInRptBuff != NULL)
        free(m_pbInRptBuff);
    if (m_pbOutRptBuff != NULL)
        free(m_pbOutRptBuff);
}


int CUsbPort::ReadData(LPBYTE pbBuff, DWORD dwBytesToRead, LPDWORD pdwBytesRead)
{
    DWORD dwBytesRead;
    DWORD dwError;
    UINT iLength = 0;
    UINT iFrameIndex = 0;
    UINT iFrameCnt = 0;
	BOOL bResult;

    while ((int)dwBytesToRead > 0)
    {
        if (!ReadFile(m_hFile, m_pbInRptBuff, m_nInRptLen, &dwBytesRead, &m_oRead))
        {
            dwError = GetLastError();
            if (dwError == ERROR_IO_PENDING)
            {
                while (TRUE)
                {
					bResult = GetOverlappedResult(m_hFile, &m_oRead, &dwBytesRead, FALSE);
					/*CString str;
					str.Format(_T("ReadData : bResult : = %d"),bResult);
					MessageBox(NULL,str,0,0);*/
                    if (bResult)
                    {
                        if (dwBytesRead != m_nInRptLen)
                            return F3_E_UNKNOWN_ERROR;
                        else
                            break;
                    } else
                    {
                        dwError = GetLastError();
                        if (dwError == ERROR_IO_PENDING ||
                            dwError == ERROR_IO_INCOMPLETE)
                        {
                            Sleep(10);
                            continue;
                        } else
                            return F3_E_UNKNOWN_ERROR;
                    }
					
                }
            } else
            {
                return F3_E_UNKNOWN_ERROR;
            }
        } else
        {
            if (m_nInRptLen != dwBytesRead)
                return F3_E_UNKNOWN_ERROR;
        }

        m_pLogger->Write(TAG_READ, m_pbInRptBuff, m_nInRptLen);

        memcpy(&pbBuff[iLength], &m_pbInRptBuff[3], m_pbInRptBuff[2]);
        iLength += m_pbInRptBuff[2];
        if (m_pbInRptBuff[1] == 0 || m_pbInRptBuff[1] == 2)
            break;
    }

    if (pdwBytesRead != NULL)
        *pdwBytesRead = iLength;

    return 0;
}


int CUsbPort::WriteData(LPBYTE pbBuff, DWORD dwBytesToWrite)
{
    DWORD dwBytesWritten;
    UINT nLength;
    UINT iOffset = 0;
    BYTE iFrameIndex = 0;
    DWORD dwError;

	CString str;
    while ((int) dwBytesToWrite > 0)
    {
        RtlZeroMemory(m_pbOutRptBuff, m_nOutRptLen);

        m_pbOutRptBuff[0] = 0;  //  ID
        nLength = min(dwBytesToWrite, m_nOutRptLen - 3);
		//str.Format(_T("m_nOutRptLen = %d  nLength = %d   dwBytesToWrite = %d "),m_nOutRptLen,nLength,dwBytesToWrite);
		//MessageBox(NULL,str,0,0);
        m_pbOutRptBuff[2] = nLength;
        memcpy(&m_pbOutRptBuff[3], &pbBuff[iOffset], nLength);
        iOffset += nLength;
        dwBytesToWrite -= nLength;
        if (dwBytesToWrite == 0)
            m_pbOutRptBuff[1] = iFrameIndex > 0 ? 2 : 0;
        else
            m_pbOutRptBuff[1] = 1;
        m_pLogger->Write(TAG_WRITE, m_pbOutRptBuff, m_nOutRptLen);

        if (!WriteFile(m_hFile, m_pbOutRptBuff, m_nOutRptLen, &dwBytesWritten, &m_oWrite))
        {
            dwError = GetLastError();
            if (dwError == ERROR_IO_PENDING)
            {
                switch (WaitForSingleObject(m_oWrite.hEvent, 1000))
                {
                case WAIT_OBJECT_0:
                    if (GetOverlappedResult(m_hFile, &m_oWrite, &dwBytesWritten, FALSE))
                    {
                        if (m_nOutRptLen != dwBytesWritten)
                            return F3_E_UNKNOWN_ERROR;
                        else
                            break;
                    } else
                        return F3_E_UNKNOWN_ERROR;

                case WAIT_TIMEOUT:
                    return F3_E_COMM_TIMEOUT;
                default:
                    return F3_E_UNKNOWN_ERROR;
                }
            } else
			{
				if(dwError == 1167)
					return 1111;
			}
        }
        iFrameIndex++;
    }
    return 0;
}

int CUsbPort::WaitDataAvailable(DWORD dwTimeOut)
{
    return 0;
    COMSTAT CommStat;
    DWORD dwCurrentTime;
    DWORD dwStartTime;
    DWORD dwElapsedTime;
    DWORD dwErrors;

    dwStartTime = GetTickCount();
    dwElapsedTime = 0;
    while (TRUE)
    {

        if (!ClearCommError(m_hFile, &dwErrors, &CommStat))
            return F3_E_UNKNOWN_ERROR;

        if (dwErrors > 0)
            return F3_E_UNKNOWN_ERROR;

        if (CommStat.cbInQue > 0)
            return 0;

        dwCurrentTime = GetTickCount();
        if ( dwCurrentTime >= dwStartTime )
            dwElapsedTime += dwCurrentTime - dwStartTime;
        else
            dwElapsedTime += 0xFFFFFFFF - dwStartTime + dwCurrentTime;

        dwStartTime = dwCurrentTime;

        if (dwElapsedTime >= dwTimeOut)
            return F3_E_UNKNOWN_ERROR;

        Sleep(50);
    }
}

int CUsbPort::ResetInput()
{
    return 0;
}


/**************************************************************
*pDevPath:·
***************************************************************/
int CUsbPort::InitPort(char *pDevPath,DWORD dwBps)
{

    m_hFile = CreateFileA(pDevPath,
                         GENERIC_READ | GENERIC_WRITE,
                         0, NULL, OPEN_EXISTING,FILE_FLAG_OVERLAPPED,NULL );
    PHIDP_PREPARSED_DATA  pPreparsedData;
    if (!HidD_GetPreparsedData(m_hFile, &pPreparsedData))
    {
        CloseHandle(m_hFile);
        return F3_E_COMM_ERROR;
    } else
    {
        HIDP_CAPS caps;
        if (HidP_GetCaps(pPreparsedData, &caps) != HIDP_STATUS_SUCCESS)
        {
            CloseHandle(m_hFile);
            HidD_FreePreparsedData(pPreparsedData);
            return F3_E_COMM_ERROR;
        } else
        {
            m_nInRptLen = caps.InputReportByteLength;
            m_nOutRptLen = caps.OutputReportByteLength;
            m_pbInRptBuff = (PBYTE) malloc (m_nInRptLen);
            m_pbOutRptBuff = (PBYTE) malloc (m_nOutRptLen);
        }
        HidD_FreePreparsedData(pPreparsedData);
    }


    return 0;
}