// I2cPanel.cpp : ʵļ
//

#include "stdafx.h"
#include "F3TEST.h"
#include "I2cPanel.h"


// CI2cPanel Ի

IMPLEMENT_DYNAMIC(CI2cPanel, CPanelBase)

CI2cPanel::CI2cPanel(CWnd* pParent /*=NULL*/)
	: CPanelBase(CI2cPanel::IDD, pParent)
{

}

CI2cPanel::~CI2cPanel()
{
}

void CI2cPanel::DoDataExchange(CDataExchange* pDX)
{
	CPanelBase::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_I2C_GRID, m_MmGrid);
	DDX_Control(pDX, IDC_I2C_ADDRESS_BOX, m_AddrBox);
	DDX_Control(pDX, IDC_I2C_BYTES_BOX, m_BytesBox);
}


BEGIN_MESSAGE_MAP(CI2cPanel, CPanelBase)
	ON_BN_CLICKED(IDC_I2C_ACTIVATE, &CI2cPanel::OnBnClickedI2cActivate)
	ON_BN_CLICKED(IDC_I2C_DEACTIVATE, &CI2cPanel::OnBnClickedI2cDeactivate)
	ON_BN_CLICKED(IDC_I2C_STATUS, &CI2cPanel::OnBnClickedI2cStatus)
	ON_BN_CLICKED(IDC_I2C_READ, &CI2cPanel::OnBnClickedI2cRead)
	ON_BN_CLICKED(IDC_I2C_WRITE, &CI2cPanel::OnBnClickedI2cWrite)
	ON_WM_ENABLE()
END_MESSAGE_MAP()


// CI2cPanel Ϣ


BOOL CI2cPanel::OnInitDialog()
{
	CPanelBase::OnInitDialog();

	m_MmGrid.Init(9, 17, 1, 1);
	m_MmGrid.InitRowHeader();
	m_MmGrid.InitColumnHeader();

	m_AddrBox.AllowHexInput();
	m_AddrBox.SetLimitText(4);
	m_AddrBox.SetRange(0, 0x7fff);
	m_AddrBox.SetValue(0);

	m_BytesBox.SetValue(128);
	m_BytesBox.EnableWindow(FALSE);

	InitCardList();

	return TRUE;  // return TRUE unless you set the focus to a control
	// 쳣: OCX ҳӦ FALSE
}


void CI2cPanel::OnBnClickedI2cActivate()
{
	CComboBox *pList = GetComboBox(IDC_I2C_CARD_TYPE_LIST);
	BYTE bCardType = (BYTE) pList->GetItemData(pList->GetCurSel());

	LONG lResult = F3_I2cActivate(g_hReader, bCardType);
	OutputResult(lResult, GetFocus(), NULL);
}


void CI2cPanel::OnBnClickedI2cDeactivate()
{
	LONG lResult = F3_I2cDeactivate(g_hReader);
	OutputResult(lResult, GetFocus(), NULL);
}


void CI2cPanel::OnBnClickedI2cStatus()
{
	BYTE bCardType;
	LONG lResult = F3_I2cGetStatus(g_hReader, &bCardType);

	CString strMsg;
	if (lResult == 0)
	{
		CString strCardName;

		CComboBox *pList = GetComboBox(IDC_I2C_CARD_TYPE_LIST);
		for (int i = 0; i < pList->GetCount(); i++)
		{
			if (bCardType == (BYTE) pList->GetItemData(i))
			{
				pList->GetLBText(i, strCardName);
				pList->SetCurSel(i);
				break;
			}
		}

		if (bCardType == I2CTYPE_UNKNOWN)
			strMsg = _T("No I2C Activated");
		else
			strMsg.Format(_T("%s Activated"), strCardName);
	}

	OutputResult(lResult, GetFocus(), strMsg);
}


void CI2cPanel::OnBnClickedI2cRead()
{
	if (! m_AddrBox.IsValidInput())
		return;
	if (! m_BytesBox.IsValidInput())
		return;

	WORD wAddr = (WORD) m_AddrBox.GetValue();
	BYTE nBytesToRead = (BYTE) m_BytesBox.GetValue();

	BYTE bBuffer[256];
	DWORD Length = sizeof(bBuffer);

	LONG lResult = F3_I2cReadMemory(g_hReader, wAddr, nBytesToRead, bBuffer, &Length);

	OutputResult(lResult, GetFocus(), NULL);

	m_MmGrid.ResetContent();

	if (lResult == 0)
		m_MmGrid.Fill(bBuffer, Length);
}


void CI2cPanel::OnBnClickedI2cWrite()
{
	if (! m_AddrBox.IsValidInput())
		return;
	if (! m_BytesBox.IsValidInput())
		return;
	if (! m_MmGrid.ValidateInput())
		return;

	WORD wAddr = (WORD) m_AddrBox.GetValue();

	BYTE bBuffer[256];
	BYTE nBytesToWrite = (BYTE) m_MmGrid.GetContent(bBuffer, sizeof(bBuffer));

	LONG lResult = F3_I2cWriteMemory(g_hReader, wAddr, nBytesToWrite, bBuffer);
	OutputResult(lResult, GetFocus(), NULL);

	if (lResult == 0)
		m_MmGrid.ResetContent();
}


void CI2cPanel::InitCardList()
{
	CComboBox *pList = GetComboBox(IDC_I2C_CARD_TYPE_LIST);

	CString s;
	LoadString(IDS_UNKNOWN, s);

	struct ITEM
	{
		LPCTSTR pszName;
		BYTE	bType;
	} items[] = {
		{ s, I2CTYPE_UNKNOWN },
		{ _T("AT24C01"), I2CTYPE_24C01 },
		{ _T("AT24C02"), I2CTYPE_24C02 },
		{ _T("AT24C04"), I2CTYPE_24C04 },
		{ _T("AT24C08"), I2CTYPE_24C08 },
		{ _T("AT24C16"), I2CTYPE_24C16 },
		{ _T("AT24C32"), I2CTYPE_24C32 },
		{ _T("AT24C64"), I2CTYPE_24C64 },
		{ _T("AT24C128"), I2CTYPE_24C128 },
		{ _T("AT24C256"), I2CTYPE_24C256 } };

	int nCount = sizeof(items) / sizeof(items[0]);
	
	int idx;
	for (int i = 0; i < nCount; i++)
	{
		idx = pList->AddString(items[i].pszName);
		pList->SetItemData(idx, items[i].bType);
	}

	pList->SetCurSel(0);
}

void CI2cPanel::OnEnable(BOOL bEnable)
{
	CPanelBase::OnEnable(bEnable);

	GetDlgItem(IDC_I2C_ACTIVATE)->EnableWindow(bEnable);
	GetDlgItem(IDC_I2C_DEACTIVATE)->EnableWindow(bEnable);
	GetDlgItem(IDC_I2C_STATUS)->EnableWindow(bEnable);
	GetDlgItem(IDC_I2C_READ)->EnableWindow(bEnable);
	GetDlgItem(IDC_I2C_WRITE)->EnableWindow(bEnable);
	GetDlgItem(IDC_I2C_CARD_TYPE_LIST)->EnableWindow(bEnable);

	m_AddrBox.EnableWindow(bEnable);
}
