// UITabControl.cpp : ʵļ
//

#include "stdafx.h"
#include "UITabControl.h"


#define PADDING  2


// UITabControl

IMPLEMENT_DYNAMIC(UITabControl, CTabCtrl)

UITabControl::UITabControl()
{
	m_tabItems.RemoveAll();
	m_iCurSel = 0;
}

UITabControl::~UITabControl()
{
}


BEGIN_MESSAGE_MAP(UITabControl, CTabCtrl)
	ON_NOTIFY_REFLECT_EX(TCN_SELCHANGE, &UITabControl::OnTcnSelchange)
	ON_WM_ENABLE()
	ON_WM_SIZE()
	ON_WM_CREATE()
END_MESSAGE_MAP()



// UITabControl Ϣ

void UITabControl::PreSubclassWindow()
{
	// TODO: ڴרô/û

	CTabCtrl::PreSubclassWindow();

	CFont *pFont = GetFont();
	if (pFont != NULL)
	{
		LOGFONT lfont;
		pFont->GetLogFont(&lfont);
		lfont.lfWeight = FW_BOLD;
		m_BoldFont.CreateFontIndirect(&lfont);
		SetFont(&m_BoldFont);
	}
}

int UITabControl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CTabCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;

	CWnd *pParent = this->GetParent();
	if (pParent != NULL)
	{
		SetFont(pParent->GetFont());
	}
	return 0;
}

BOOL UITabControl::OnTcnSelchange(NMHDR *pNMHDR, LRESULT *pResult)
{
	if (m_tabItems.GetCount() == 0)
		return FALSE;

	TABITEM tabitem = m_tabItems[m_iCurSel];
	tabitem.pWnd->ShowWindow(SW_HIDE);
	
	m_iCurSel = GetCurSel();
	tabitem = m_tabItems[m_iCurSel];
	tabitem.pWnd->ShowWindow(SW_SHOW);
	tabitem.pWnd->SetFocus();

	*pResult = 0;

	return FALSE;
}


void UITabControl::OnSize(UINT nType, int cx, int cy)
{
	int nCount = GetItemCount();
	for (int i = 0; i < nCount; i++)
		RelocTabPage(i, m_tabItems[i].pWnd, i == m_iCurSel);
}


void UITabControl::OnEnable(BOOL bEnable)
{
	CWnd *pWnd;
	int nCount = GetItemCount();
	for (int i = 0; i < nCount; i++)
	{
		pWnd = m_tabItems[i].pWnd;
		if (pWnd != NULL)
			pWnd->EnableWindow(bEnable);
	}
}


BOOL UITabControl::Create(CWnd *pParentWnd, UINT nID)
{
    ASSERT(pParentWnd != NULL);

    CRect rect;
    CWnd *pWnd = pParentWnd->GetDlgItem(nID);
    if (pWnd != NULL)
    {
        pWnd->GetWindowRect(&rect);
        pParentWnd->ScreenToClient(&rect);
    }

    return CWnd::Create(NULL, NULL, WS_VISIBLE| WS_CHILD, rect, pParentWnd, nID);
}


int UITabControl::AddTabPage(CWnd *pWnd, LPCTSTR pszLabel)
{
	ASSERT(pWnd != NULL);
	ASSERT(pszLabel != NULL);

	int nItem;
	pWnd->SetWindowText(pszLabel);
	pWnd->SetParent(this);
	nItem = (int) CTabCtrl::InsertItem(GetItemCount(), pszLabel);

	TABITEM tabitem = { pWnd, NULL };
	m_tabItems.SetAtGrow(nItem, tabitem);
	RelocTabPage(nItem, pWnd, nItem == m_iCurSel);

	return nItem;
}


void UITabControl::SetTabData(int iTab, DWORD_PTR dwData)
{
	ASSERT(iTab < m_tabItems.GetCount());
	m_tabItems[iTab].dwData = dwData;
}


DWORD_PTR UITabControl::GetTabData(int iTab)
{
	ASSERT(iTab < m_tabItems.GetCount());
	return m_tabItems[iTab].dwData;
}


void UITabControl::Select(CWnd *pWnd)
{
	int nCount = GetItemCount();
	for (int i = 0; i < nCount; i++)
	{
		if (m_tabItems[i].pWnd == pWnd)
		{
			LRESULT result;
			SetCurSel(i);
			OnTcnSelchange(NULL, &result);
			break;
		}
	}
}


void UITabControl::RelocTabPage(int nItem, CWnd *pWnd, BOOL bShow)
{
	int nBorderHeight = GetSystemMetrics(SM_CYFRAME);
	int nBorderWidth = GetSystemMetrics(SM_CXFRAME);

	CRect itemRect;
	GetItemRect(nItem, &itemRect);

	CRect clientRect;
	GetClientRect(&clientRect);

	clientRect.left += nBorderWidth + PADDING;
	clientRect.right -= nBorderWidth  + PADDING;
	clientRect.top += itemRect.Height() + nBorderHeight + PADDING;
	clientRect.bottom -= nBorderHeight + PADDING;

	pWnd->SetWindowPos(NULL, 
		clientRect.left, 
		clientRect.top, 
		clientRect.Width(), 
		clientRect.Height(),
		bShow ? SWP_SHOWWINDOW : SWP_HIDEWINDOW);
}